@@ -8,7 +8,9 @@ debug_msg <- function(...) {
 assert_work_queue_pkgs <- local({
   val <- NULL
   function() {
-    if (!is.null(val)) return()
+    if (!is.null(val)) {
+      return()
+    }
     for (pkg in list(
       list(name = "future", version = "1.21.0"),
       list(name = "fastmap", version = "1.1.0")
@@ -114,7 +116,6 @@ ExpoDelay <- R6::R6Class(
 # FIFO queue of workers
 #' Future promise work queue
 #'
-#' #' `r lifecycle::badge('experimental')`
 #'
 #' An \pkg{R6} class to help with scheduling work to be completed. `WorkQueue` will only execute work if the `can_proceed()` returns `TRUE`. For the use case of `future`, `can_proceed()` defaults to `future::nbrOfFreeWorkers() > 0` which will not allow for work to be executed if a \pkg{future} worker is not available.
 #'
@@ -131,7 +132,7 @@ ExpoDelay <- R6::R6Class(
 #'
 #' However, it is possible to use a private loop inside a user-defined `WorkQueue` may work which can be provided directly to `future_promise(queue=custom_queue)`. Having a concrete example (or need) will help us understand the problem better. If you have an example, please reach out .
 #'
-#' @seealso [future_promise_queue()] which returns a `WorkQueue` which is cached per R session.
+#' @seealso [`promises::future_promise_queue()`] which returns a `WorkQueue` which is cached per R session.
 #' @keywords internal
 WorkQueue <- R6::R6Class(
   "WorkQueue",
@@ -192,7 +193,9 @@ WorkQueue <- R6::R6Class(
     can_proceed = function() {
       can_proceed_cache_val <- private$can_proceed_cache_val
       # Return early if no work can be submitted.
-      if (!is.null(can_proceed_cache_val)) return(can_proceed_cache_val)
+      if (!is.null(can_proceed_cache_val)) {
+        return(can_proceed_cache_val)
+      }
 
       ret <- isTRUE(private$can_proceed_fn())
 
@@ -222,7 +225,9 @@ WorkQueue <- R6::R6Class(
     attempt_work = function(can_delay = FALSE) {
       debug_msg('attempt_work()')
       # If nothing to start, return early
-      if (private$queue$size() == 0) return()
+      if (private$queue$size() == 0) {
+        return()
+      }
 
       # If we are not waiting on someone else, we can do work now
       while ((private$queue$size() > 0) && private$can_proceed()) {
@@ -258,7 +263,9 @@ WorkQueue <- R6::R6Class(
 
       # Safety check...
       # If nothing is returned, no work to be done. Return early
-      if (!is.function(work_fn)) return()
+      if (!is.function(work_fn)) {
+        return()
+      }
 
       # Do scheduled work
       debug_msg("execute work")
@@ -328,8 +335,8 @@ WorkQueue <- R6::R6Class(
 )
 
 
-#' @describeIn future_promise Default `future_promise()` work queue to use. This function returns a [WorkQueue] that is cached per R session.
-#' @seealso [`WorkQueue`]
+#' @describeIn future_promise Default `future_promise()` work queue to use. This function returns a [`promises::WorkQueue`] that is cached per R session.
+#' @seealso [`promises::WorkQueue`]
 #' @export
 future_promise_queue <- local({
   future_promise_queue_ <- NULL
@@ -345,7 +352,6 @@ future_promise_queue <- local({
 
 #' \pkg{future} promise
 #'
-#' `r lifecycle::badge('experimental')`
 #'
 #' When submitting \pkg{future} work, \pkg{future} (by design) will block the main R session until a worker becomes available.
 #' This occurs when there is more submitted \pkg{future} work than there are available \pkg{future} workers.
@@ -353,15 +359,15 @@ future_promise_queue <- local({
 #'
 #' Using `future_promise()` is recommended whenever a continuous runtime is used, such as with \pkg{plumber} or \pkg{shiny}.
 #'
-#' For more details and examples, please see the [`vignette("future_promise", "promises")`](https://rstudio.github.io/promises/articles/future_promise.html) vignette.
+#' For more details and examples, please see the [`vignette("future_promise", "promises")`](https://rstudio.github.io/promises/articles/promises_05b_future_promise.html) vignette.
 #' @describeIn future_promise Creates a [promise()] that will execute the `expr` using [future::future()].
 #' @inheritParams future::future
 #' @param expr An R expression. While the `expr` is eventually sent to [`future::future()`], please use the same precautions that you would use with regular `promises::promise()` expressions. `future_promise()` may have to hold the `expr` in a [promise()] while waiting for a \pkg{future} worker to become available.
 #' @param ... extra parameters provided to [`future::future()`]
-#' @param queue A queue that is used to schedule work to be done using [future::future()].  This queue defaults to [future_promise_queue()] and requires that method `queue$schedule_work(fn)` exist.  This method should take in a function that will execute the promised \pkg{future} work.
+#' @param queue A queue that is used to schedule work to be done using [future::future()].  This queue defaults to [`promises::future_promise_queue()`] and requires that method `queue$schedule_work(fn)` exist.  This method should take in a function that will execute the promised \pkg{future} work.
 #' @return Unlike [`future::future()`], `future_promise()` returns a [promise()] object that will eventually resolve the \pkg{future} `expr`.
 #' @examples
-#' \donttest{# Relative start time
+#' \dontrun{# Relative start time
 #' start <- Sys.time()
 #' # Helper to force two `future` workers
 #' with_two_workers <- function(expr) {
@@ -369,7 +375,7 @@ future_promise_queue <- local({
 #'     message("`future` not installed")
 #'     return()
 #'   }
-#'   old_plan <- future::plan(future::multisession(workers = 2))
+#'   old_plan <- future::plan(future::multisession, workers = 2)
 #'   on.exit({future::plan(old_plan)}, add = TRUE)
 #'   start <<- Sys.time()
 #'   force(expr)
@@ -388,8 +394,12 @@ future_promise_queue <- local({
 #' # `"promise done"` will appear after four workers are done and the main R session is not blocked
 #' # The important thing to note is the first four times will be roughly the same
 #' with_two_workers({
-#'   promise_resolve(Sys.getpid()) %...>% print_msg("promise done")
-#'   for (i in 1:6) future::future({Sys.sleep(1); Sys.getpid()}) %...>% print_msg("future done")
+#'   promise_resolve(Sys.getpid()) |>
+#'     then(\(x) {print_msg("promise done")})
+#'   for (i in 1:6) {
+#'     future::future({Sys.sleep(1); Sys.getpid()}) |>
+#'       then(\(x) {print_msg("future done")})
+#'   }
 #' })
 #' {
 #' #> PID: XXX; 2.5s promise done
@@ -404,8 +414,12 @@ future_promise_queue <- local({
 #' # `"promise done"` will almost immediately, before any workers have completed
 #' # The first two `"future done"` comments appear earlier the example above
 #' with_two_workers({
-#'   promise_resolve(Sys.getpid()) %...>% print_msg("promise")
-#'   for (i in 1:6) future_promise({Sys.sleep(1); Sys.getpid()}) %...>% print_msg("future done")
+#'   promise_resolve(Sys.getpid()) |>
+#'     then(\(x) {print_msg("promise")})
+#'   for (i in 1:6) {
+#'     future_promise({Sys.sleep(1); Sys.getpid()}) |>
+#'       then(\(x) {print_msg("future done")})
+#'   }
 #' })
 #' {
 #' #> PID: XXX; 0.2s promise done
@@ -430,7 +444,9 @@ future_promise <- function(
       length(formals(queue$schedule_work)) >= 1
   )
 
-  if (substitute) expr <- substitute(expr)
+  if (substitute) {
+    expr <- substitute(expr)
+  }
 
   # Force all variables to curb values changing before execution
   # Does NOT fix R environment values changing
@@ -458,131 +474,3 @@ future_promise <- function(
     })
   })
 }
-
-
-if (FALSE) {
-  # ConstDelay <- R6::R6Class("ConstDelay",
-  #   inherit = Delay,
-  #   private = list(
-  #     const = 0.1,
-  #     random = TRUE
-  #   ),
-  #   public = list(
-  #     initialize = function(const = 0.1, random = TRUE) {
-  #       stopifnot(length(const) == 1 && is.numeric(const) && const >= 0)
-  #       private$const <- const
-  #       private$random <- isTRUE(random)
-
-  #       self
-  #     },
-  #     delay = function() {
-  #       if (private$random) {
-  #         runif(n = 1, max = private$const)
-  #       } else {
-  #         private$const
-  #       }
-  #     }
-  #   )
-  # )
-  # LinearDelay <- R6::R6Class("LinearDelay",
-  #   inherit = Delay,
-  #   private = list(
-  #     delta = 0.05,
-  #     random = TRUE
-  #   ),
-  #   public = list(
-  #     initialize = function(delta = 0.03, random = TRUE) {
-  #       stopifnot(length(delta) == 1 && is.numeric(delta) && delta >= 0)
-  #       private$delta <- delta
-  #       private$random <- isTRUE(random)
-
-  #       self
-  #     },
-
-  #     delay = function() {
-  #       delta_delay <- private$delay_count * private$delta
-  #       if (private$random) {
-  #         runif(n = 1, max = delta_delay)
-  #       } else {
-  #         delta_delay
-  #       }
-  #     }
-  #   )
-  # )
-
-  # dev_load <- pkgload::load_all
-
-  # ## test
-  # dev_load(); print_i(); start <- Sys.time(); promise_all(.list = lapply(1:10, function(x) { future_promise({ Sys.sleep(1); print(paste0(x)) })})) %...>% { print(Sys.time() - start) };
-
-  # ## block workers mid job
-  # dev_load(); print_i(); start <- Sys.time(); promise_all(.list = lapply(1:10, function(x) { future_promise({ Sys.sleep(1); print(paste0(x)) })})) %...>% { print(Sys.time() - start) }; lapply(1:2, function(i) { later::later(function() { message("*************** adding blockage", i); fj <- future::future({ Sys.sleep(4); message("*************** blockage done", i); i}); then(fj, function(x) { print(paste0("block - ", i))}); }, delay = 0.5 + i) }) -> ignore;
-
-  # ## block main worker mid job
-  # dev_load(); print_i(); start <- Sys.time(); promise_all(.list = lapply(1:10, function(x) { future_promise({ Sys.sleep(1); print(paste0(x)) })})) %...>% { print(Sys.time() - start) }; lapply(1:4, function(i) { later::later(function() { message("*************** adding blockage", i); fj <- future::future({ Sys.sleep(4); message("*************** blockage done", i); i}); then(fj, function(x) { print(paste0("block - ", i))}); }, delay = 0.5 + i/4) }) -> ignore;
-
-  # ## block workers pre job
-  # dev_load(); print_i(); lapply(1:2, function(i) { message("*************** adding blockage", i); future::future({ Sys.sleep(4); message("*************** blockage done", i); i}) }) -> future_jobs; lapply(future_jobs, function(fj) { as.promise(fj) %...>% { print(.) } }); start <- Sys.time(); promise_all(.list = lapply(1:10, function(x) { future_promise({ Sys.sleep(1); print(paste0(x)) })})) %...>% { print(Sys.time() - start) };
-
-  # ## block main worker workers pre job
-  # dev_load(); print_i(); start <- Sys.time(); promise_all(.list = lapply(1:10, function(x) { future_promise({ Sys.sleep(1); print(paste0(x)) })})) %...>% { print(Sys.time() - start) }; lapply(1:4, function(i) { later::later(function() { message("*************** adding blockage", i); fj <- future::future({ Sys.sleep(4); message("*************** blockage done", i); i}); then(fj, function(x) { print(paste0("block - ", i))}); }, delay = 0.5 + i/4) }) -> ignore;
-
-  future::plan(future::multisession(workers = 2))
-
-  debug_msg_can_print <- TRUE
-
-  print_i <- function(i = 0) {
-    if (i <= 50) {
-      print(i)
-      later::later(
-        function() {
-          print_i(i + 1)
-        },
-        delay = 0.1
-      )
-    }
-  }
-
-  slow_calc <- function(n) {
-    Sys.sleep(n)
-    "slow!"
-  }
-  n <- 2
-  prom <- future_promise
-  # prom <- future::future
-  a1 <- prom({
-    print(paste0("start 1 - ", Sys.time()))
-    print(slow_calc(n))
-  })
-  a2 <- prom({
-    print(paste0("start 2 - ", Sys.time()))
-    print(slow_calc(n))
-  })
-  a3 <- prom({
-    print(paste0("start 3 - ", Sys.time()))
-    print(slow_calc(n))
-  })
-  a4 <- prom({
-    print(paste0("start 4 - ", Sys.time()))
-    print(slow_calc(n))
-  })
-
-  print("done assignement!")
-
-  a1 %...>%
-    {
-      message("end 1 - ", format(Sys.time()))
-    }
-  a2 %...>%
-    {
-      message("end 2 - ", format(Sys.time()))
-    }
-  a3 %...>%
-    {
-      message("end 3 - ", format(Sys.time()))
-    }
-  a4 %...>%
-    {
-      message("end 4 - ", format(Sys.time()))
-    }
-}
