@@ -1,5 +1,3 @@
-#' @useDynLib promises
-#' @importFrom Rcpp evalCpp
 #' @import later
 NULL
 
@@ -87,7 +85,7 @@ Promise <- R6::R6Class(
         private$onRejected <- list()
 
         later::later(
-          ~ {
+          function() {
             if (!private$rejectionHandled) {
               # warning() was unreliable here
               cat(
@@ -110,7 +108,9 @@ Promise <- R6::R6Class(
     },
     resolve = function(value) {
       # Only allow this to be called once, then no-op.
-      if (private$publicResolveRejectCalled) return(invisible())
+      if (private$publicResolveRejectCalled) {
+        return(invisible())
+      }
       private$publicResolveRejectCalled <- TRUE
 
       tryCatch(
@@ -130,7 +130,9 @@ Promise <- R6::R6Class(
     },
     reject = function(reason) {
       # Only allow this to be called once, then no-op.
-      if (private$publicResolveRejectCalled) return(invisible())
+      if (private$publicResolveRejectCalled) {
+        return(invisible())
+      }
       private$publicResolveRejectCalled <- TRUE
 
       tryCatch(
@@ -223,7 +225,9 @@ Promise <- R6::R6Class(
         "<Promise [pending]>"
       } else {
         classname <- class(private$value)[[1]]
-        if (length(classname) == 0) classname <- ""
+        if (length(classname) == 0) {
+          classname <- ""
+        }
 
         sprintf("<Promise [%s: %s]>", private$state, classname)
       }
@@ -302,40 +306,53 @@ normalizeOnRejected <- function(onRejected) {
 #'
 #' The return value of `action` will be ignored.
 #'
-#' @param action A function with signature `function(resolve, reject)`, or a
-#'   one-sided formula. See Details.
+#' @section `action=` formulas:
+#'
+#' `r lifecycle::badge("superseded")`
+#'
+#' With `{promises}` depending on R >= 4.1, the shorthand of a formula, `~
+#' fn(.)` for `action` is no longer recommended by the `{promises}` package or
+#' tidyverse (for example,
+#' [`{purrr}`](https://github.com/tidyverse/purrr/commit/670c3ed9920f15da0d4175068ecddc41f0f1f335#diff-c4dcc43795da5c7f6bf5a94d957b5507ce795fedd6d3eb092ccad03678c4f76dR15))
+#' as we now have access to the function shorthand, `\(x) fn(x)`. Please update
+#' your `action` code to use the new function shorthand syntax `\(resolve,
+#' reject) resolve(arg1, arg2)` instead of `~ { resolve(arg1, arg2) }`. The
+#' magically created `resolve`/`reject` functions can be confusing when chained
+#' with other methods.
+#'
+#' @param action A function with signature `function(resolve, reject)`.
 #'
 #' @return A promise object (see \code{\link{then}}).
 #'
 #' @examples
 #' # Create a promise that resolves to a random value after 2 secs
-#' p1 <- promise(function(resolve, reject) {
-#'   later::later(~resolve(runif(1)), delay = 2)
+#' p1 <- promise(\(resolve, reject) {
+#'   later::later(\() resolve(runif(1)), delay = 2)
 #' })
 #'
-#' p1 %...>% print()
+#' p1 |> then(print)
 #'
 #' # Create a promise that errors immediately
-#' p2 <- promise(~{
+#' p2 <- promise(\(resolve, reject) {
 #'   reject("An error has occurred")
 #' })
 #' then(p2,
-#'   onFulfilled = ~message("Success"),
-#'   onRejected = ~message("Failure")
+#'   onFulfilled = \(value) message("Success"),
+#'   onRejected = \(err) message("Failure")
 #' )
 #'
 #' @export
 promise <- function(action) {
-  if (inherits(action, "formula")) {
-    if (length(action) != 2) {
-      stop("'action' must be a function or one-sided formula")
-    }
-  } else if (is.function(action)) {
+  if (is.function(action)) {
     if (length(formals(action)) != 2) {
       stop("'action' function must have two arguments")
     }
+  } else if (inherits(action, "formula")) {
+    if (length(action) != 2) {
+      stop("'action' must be a function or one-sided formula")
+    }
   } else {
-    stop("Invalid action argument--must be a function or formula")
+    stop("Invalid action argument--should be a function")
   }
 
   p <- Promise$new()
@@ -390,22 +407,22 @@ promise <- function(action) {
 #' @param reason An error message string, or error object.
 #'
 #' @examples
-#' promise_resolve(mtcars) %...>%
-#'   head() %...>%
-#'   print()
+#' promise_resolve(mtcars) |>
+#'   then(head) |>
+#'   then(print)
 #'
-#' promise_reject("Something went wrong") %...T!%
-#'   { message(conditionMessage(.)) }
+#' promise_reject("Something went wrong") |>
+#'   catch(tee = TRUE, \(e) message(conditionMessage(e)))
 #'
 #' @export
 promise_resolve <- function(value) {
-  promise(~ resolve(value))
+  promise(function(resolve, reject) resolve(value))
 }
 
 #' @rdname promise_resolve
 #' @export
 promise_reject <- function(reason) {
-  promise(~ reject(reason))
+  promise(function(resolve, reject) reject(reason))
 }
 
 #' Coerce to a promise
@@ -413,7 +430,8 @@ promise_reject <- function(reason) {
 #' Use `is.promise` to determine whether an R object is a promise. Use
 #' `as.promise` (an S3 generic method) to attempt to coerce an R object to a
 #' promise, and `is.promising` (another S3 generic method) to test whether
-#' `as.promise` is supported. This package includes support for converting
+#' `as.promise` is supported. [mirai::mirai] objects have an `as.promise` method
+#' defined in the mirai package, and this package provides one for converting
 #' [future::Future] objects into promises.
 #'
 #' @param x An R object to test or coerce.
@@ -470,6 +488,15 @@ as.promise.Future <- function(x) {
   # We want to create a promise only once for each Future object, and cache it
   # as an attribute. This spares us from having multiple polling loops waiting
   # for the same Future.
+  #
+  ## Note: 2025-07-barret:
+  ## Active bindings will not work as `{future}` does not proactively retrieve results. A poll must be used.
+  #
+  ## Note: 2025-08-barret:
+  ## From conversation with Henrik, while it may be possible to use `immediateCondition` within future to _immediately_ send a value back from a worker to the main R session,
+  ## `future::resolved()` is still required to pull the _immediate_ value into the R session (even when the future has not fully resolved yet).
+  ## Therefore, the polling mechanism below is still required.
+  ## For comparison, mirai has a background process that waits for results and will interrupt the main R session with the value.
 
   cached <- attr(x, "converted_promise", exact = TRUE)
   if (!is.null(cached)) {
