@@ -1,11 +1,17 @@
 describe("C++ interface", {
+  env <- new.env()
+  Rcpp::sourceCpp(
+    system.file("promise_task.cpp", package = "promises"),
+    env = env
+  )
+
   it("basically works", {
     promise(function(resolve, reject) {
-      asyncFib(resolve, reject, 3)
-    }) %...>%
-      {
-        expect_identical(., 2)
-      } %>%
+      env$asyncFib(resolve, reject, 3)
+    }) |>
+      then(\(x) {
+        expect_identical(x, 2)
+      }) |>
       wait_for_it()
   })
 
@@ -15,17 +21,17 @@ describe("C++ interface", {
     expect_true(is.null(current_promise_domain()))
     with_promise_domain(cd, {
       promise(function(resolve, reject) {
-        asyncFib(resolve, reject, 3)
-      }) %...>%
-        {
-          expect_identical(., 2)
+        env$asyncFib(resolve, reject, 3)
+      }) |>
+        then(\(x) {
+          expect_identical(x, 2)
           expect_identical(cd$counts$onFulfilledCalled, 1L)
-          promise_resolve(TRUE) %...>%
-            {
+          promise_resolve(TRUE) |>
+            then(\(y) {
               expect_true(!is.null(current_promise_domain()))
               expect_identical(cd$counts$onFulfilledCalled, 2L)
-            }
-        } %>%
+            })
+        }) |>
         wait_for_it()
     })
   })
